/* Copyright (C) 2004 - 2007  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
package com.db4o.internal.cs.messages;

import com.db4o.internal.*;


public class MsgObject extends MsgD {
	
	private static final int LENGTH_FOR_ALL = Const4.ID_LENGTH + (Const4.INT_LENGTH * 3);
	private static final int LENGTH_FOR_FIRST = LENGTH_FOR_ALL;
	
	private int _id;
	private int _address;
	
	MsgD getWriter(StatefulBuffer bytes, int[] prependInts) {
		int lengthNeeded = bytes.getLength() + LENGTH_FOR_FIRST;
		if(prependInts != null){
			lengthNeeded += (prependInts.length * Const4.INT_LENGTH);
		}
		int embeddedCount = bytes.embeddedCount();
		if(embeddedCount > 0){
			lengthNeeded += (LENGTH_FOR_ALL * embeddedCount) + bytes.embeddedLength();
		}
		MsgD message = getWriterForLength(bytes.getTransaction(), lengthNeeded);
		if(prependInts != null){
		    for (int i = 0; i < prependInts.length; i++) {
		        message._payLoad.writeInt(prependInts[i]);    
            }
		}
		message._payLoad.writeInt(embeddedCount);
		bytes.appendTo(message._payLoad, -1);
		return message;
	}

	public MsgD getWriter(StatefulBuffer bytes) {
		return getWriter(bytes, null);
	}
	
	public MsgD getWriter(ClassMetadata a_yapClass, StatefulBuffer bytes) {
        if(a_yapClass == null){
            return getWriter(bytes, new int[]{0});
        }
		return getWriter(bytes, new int[]{ a_yapClass.getID()});
	}
	
	public MsgD getWriter(ClassMetadata a_yapClass, int a_param, StatefulBuffer bytes) {
		return getWriter(bytes, new int[]{ a_yapClass.getID(), a_param});
	}
	
	public final StatefulBuffer unmarshall() {
		return unmarshall(0);
	}

	public final StatefulBuffer unmarshall(int addLengthBeforeFirst) {
		_payLoad.setTransaction(transaction());
		int embeddedCount = _payLoad.readInt();
		int length = _payLoad.readInt();
		if (length == 0) {
			return null;  // does this happen ?
		}
		_id = _payLoad.readInt();
		_address = _payLoad.readInt();
		if(embeddedCount == 0){
			_payLoad.removeFirstBytes(LENGTH_FOR_FIRST + addLengthBeforeFirst);
		}else{
			_payLoad._offset += length;
			StatefulBuffer[] embedded = new StatefulBuffer[embeddedCount + 1];
			embedded[0] = _payLoad;
			new StatefulBuffer(_payLoad, embedded, 1);  // this line cascades and adds all embedded YapBytes 
			_payLoad.trim4(LENGTH_FOR_FIRST + addLengthBeforeFirst, length);
		}
		_payLoad.useSlot(_id, _address, length);
		return _payLoad;
	}

	public int getId() {
		return _id;
	}
}
