/* Copyright (C) 2004 - 2007  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
package db4ounit.extensions;

import java.lang.reflect.Method;

import com.db4o.foundation.ArgumentNullException;

import db4ounit.*;

public class Db4oTestSuiteBuilder extends ReflectionTestSuiteBuilder {
	
	private static final class Db4oLabelProvider implements TestMethod.LabelProvider {
		
		public static final TestMethod.LabelProvider DEFAULT = new Db4oLabelProvider();
		
		public String getLabel(TestMethod method) {
			return "[" + fixtureLabel(method) + "] " + TestMethod.DEFAULT_LABEL_PROVIDER.getLabel(method);
		}

		private String fixtureLabel(TestMethod method) {
			return ((AbstractDb4oTestCase)method.getSubject()).fixture().getLabel();
		}
	}	
	
	private Db4oFixture _fixture;
    
	public Db4oTestSuiteBuilder(Db4oFixture fixture, Class clazz) {		
		super(clazz);
        setFixture(fixture);
	}
    
    public Db4oTestSuiteBuilder(Db4oFixture fixture, Class[] classes) {     
        super(classes);
        setFixture(fixture);
    }
    
    private void setFixture(Db4oFixture fixture){
        if (null == fixture) throw new ArgumentNullException("fixture");     
        _fixture = fixture;
    }

    protected boolean isApplicable(Class clazz) {
    	return _fixture.accept(clazz);
    }
    
	protected Object newInstance(Class clazz) {
		Object instance = super.newInstance(clazz);
		if (instance instanceof AbstractDb4oTestCase) {
			((AbstractDb4oTestCase)instance).fixture(_fixture);
		}
		return instance;
	}
	
	protected Test createTest(Object instance, Method method) {
		if (instance instanceof AbstractDb4oTestCase) {
			return new TestMethod(instance, method, Db4oLabelProvider.DEFAULT); 
		}
		return super.createTest(instance, method);
	}
}