/* Copyright (C) 2004 - 2007  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
package db4ounit;

import java.lang.reflect.Method;
import java.util.Vector;

public class ReflectionTestSuiteBuilder implements TestSuiteBuilder {
	
	private Class[] _classes;
	
	public ReflectionTestSuiteBuilder(Class clazz) {
		if (null == clazz) throw new IllegalArgumentException("clazz");
		_classes = new Class[] { clazz };
	}
	
	public ReflectionTestSuiteBuilder(Class[] classes) {
		if (null == classes) throw new IllegalArgumentException("classes");
		_classes = classes;
	}
	
	public TestSuite build() {
		return (1 == _classes.length)
			? fromClass(_classes[0])
			: fromClasses(_classes);
	}
	
	protected TestSuite fromClasses(Class[] classes) {		
		Vector suites = new Vector(classes.length);
		for (int i = 0; i < classes.length; i++) {
			TestSuite suite = fromClass(classes[i]);
			if (suite.getTests().length>0) {
				suites.addElement(suite);
			}
		}
		return new TestSuite(toTestArray(suites));
	}
	
	protected TestSuite fromClass(Class clazz) {
		if(!isApplicable(clazz)) {
			TestPlatform.emitWarning("DISABLED: " + clazz.getName());
			return new TestSuite(new Test[0]);
		}
		if(TestSuiteBuilder.class.isAssignableFrom(clazz)) {
			return ((TestSuiteBuilder)newInstance(clazz)).build();
		}
		if (Test.class.isAssignableFrom(clazz)) {
			return new TestSuite(clazz.getName(), new Test[] { (Test)newInstance(clazz) });
		}
		if (!(TestCase.class.isAssignableFrom(clazz))) {
			throw new IllegalArgumentException("" + clazz + " is not marked as " + TestCase.class);
		}
		return fromMethods(clazz);
	}

	protected boolean isApplicable(Class clazz) {
		return clazz != null; // just removing the 'parameter not used' warning
	}
	
	private TestSuite fromMethods(Class clazz) {
		Vector tests = new Vector();
		Method[] methods = clazz.getMethods();
		for (int i = 0; i < methods.length; i++) {
			Object instance=newInstance(clazz);
			Method method = methods[i];
			if (!isTestMethod(method)) {
				emitWarningOnIgnoredTestMethod(instance, method);
				continue;			
			}
			tests.addElement(createTest(instance, method));
		}		
		return new TestSuite(clazz.getName(), toTestArray(tests));
	}
	
	private void emitWarningOnIgnoredTestMethod(Object subject, Method method) {
		if (!startsWithIgnoreCase(method.getName(), "_test")) {
			return;
		}
		TestPlatform.emitWarning("IGNORED: " + createTest(subject, method).getLabel());
	}

	protected boolean isTestMethod(Method method) {
		return hasTestPrefix(method)
			&& TestPlatform.isPublic(method)
			&& !TestPlatform.isStatic(method)
			&& !TestPlatform.hasParameters(method);
	}

	private boolean hasTestPrefix(Method method) {
		return startsWithIgnoreCase(method.getName(), "test");
	}

	protected boolean startsWithIgnoreCase(final String s, final String prefix) {
		return s.toUpperCase().startsWith(prefix.toUpperCase());
	}
	
	private static Test[] toTestArray(Vector tests) {
		Test[] array = new Test[tests.size()];
		tests.copyInto(array);
		return array;
	}

	protected Object newInstance(Class clazz) {		
		try {
			return clazz.newInstance();
		} catch (Exception e) {
			throw new TestException(e);
		}
	}
	
	protected Test createTest(Object instance, Method method) {
		return new TestMethod(instance, method);
	}	
}
